' VGA to JuiceBox picture driver #1
'Copyright 2007 Raymond Allen

CON
  JX=240  'JuiceBox horizontal pixels
  JY=160  'JuiceBox vertical pixels
  JLineBytes=jx*3/2  '=360 bytes required per horizontal line (1.5 bytes per pixel)
  JLineLongs=JLineBytes/4

VAR

  long cog
  long stack[100]
  long BasePin
  'input to assembly
  long arow  'assembly routine will monitor this        
  long apscreen
  long apvga_colorbase
  long ancols
  long aBuffer
  long JHBuffer[JLineLongs]  'Juicebox picture horizontal line buffer  

OBJ

  sdfat: "fsrw"   

PUB Start(SD_BasePin, pscreen, pvga_colorbase, ncols, nrows) : okay | i 
{{Load assembly horizontal line generator into a new cog.}}
'Also starts SD card monitor and writer

  stop  'I guess make sure not already loaded
  arow:=-1  'assembly routine will watch this variable for changes
  apscreen:=pscreen
  apvga_colorbase:=pvga_colorbase
  ancols:=ncols
  aBuffer:=@JHBuffer

    
  'Start a cog do to quick line-by-line screen shot convert to Juicebox bitmap format
  okay := cog := cognew(@JuiceLine, @arow) + 1

  'Start a cog to monitor SD card activity and update after activity
  BasePin:=SD_BasePin
  cognew(JuiceMonitor,@stack)


  


PRI JuiceMonitor|phase,OffCount,beenActive, ActiveCnt                                                     
   'Monitor one of the SD card pins for activity (assuming Juicebox in slideshow mode) and update screenshot after each activity

   'Give the juicebox a few seconds to fire up
   waitcnt(cnt+clkfreq*10)

   'Setup the counter module to count clock pulses
   CTRA := (%01010 << 26) + 1  'Pos Edge count on SD card CLK
   frqa:=1

   'using LEDs on PINS P14,P15 to monitor status
   dira[14]~~
   dira[15]~~

   ActiveCnt:=0  'don't want to write on first activity

   repeat 'watch and write loop
      beenActive:=0  'init flag to indicate first activity
      phase:=PHSA  'SD clock count
      'watch loop
      repeat
        waitcnt(cnt+100_000)
        'p:=ina[1]
        if (PHSA<>phase)
          outa[14]~~  'Light LED to show activity
          phase:=PHSA
          OffCount:=0  'reset count
          beenActive:=1
        else
          outa[14]~  'turn LED off
          OffCount++  'increment inactivity counter
        if (beenActive>0) AND (OffCount>100)  'if a long period in inactivity after activity
          ActiveCnt++   
          if (ActiveCnt>2)
            quit  'break inner repeat loop
      outa[15]~~  'turn on Prop SD write indicator
      SavePicture(BasePin)  'make and save screen as .jbp bitmap 
      outa[15]~  'turn off write indicator
      'repeat
'
   
{  dira[15]~~
  repeat
    !outa[15]
    waitcnt(3_000_000 + cnt)
    }

   

PRI SavePicture(SD_BasePin)|i
     


  sdfat.mount(SD_BasePin) 'start the SD driver
  'give assembly a moment to load
  waitcnt(cnt+100000)

  sdfat.popen(string("Prop.jbp"), "w")     'open the file
  repeat i from 0 to jy-1
      'signal assembly routine to generate pixel line data
      arow:=i
      'give assembly a moment to react
      waitcnt(cnt+100000)

      'write line data
      sdfat.pwrite(@JHBuffer,JLineBytes)    'write line
     
  sdfat.pclose
  sdfat.stop        'stop the SD driver so it doesn't interfere with the JuiceBox

  
PUB stop

'' Stop driver - frees a cog

  if cog
    cogstop(cog~ -  1)
      

DAT
        ORG 0
JuiceLine     'generate a JuiceBox picture horizontal line given row#, monitor row# and fill buffer with new data when changed
'load 4 input parameters from hub to cog given address in par
        movd    :par,#jrow             
        mov     x,par
        mov     y,#5  'input 5 parameters
:par    rdlong  0,x
        add     :par,dlsb
        add     x,#4
        djnz    y,#:par

        'watch row variable and wait for change
        mov   lastrow,jrow
waitRow rdlong jrow,par
        cmp  lastrow,jrow wz
        IF_Z jmp #waitRow

        'input row# has changed, so write pixel data
        mov y, jbuffer       'address to write to
        mov x, #JLineLongs   'number of long writes required to generate a horizontal line
        'mov jx, #0  'start at leftmost pixel (#0)
        mov ntiles, #30 'assuming 30 tile column display
        mov xtile, #0  'start at tile #0, and at tile #29

TileLoop  '1.5 bytes required per pixel...  So, 3 bytes for 2 pixels, 6 bytes for 4 pixels, 12 bytes=3longs for 8 pixels
        'each tile is 32 bits wide with 2 bit color for 16 pixels per tile
        'but, we are only using every other pixel (assuming 30 cols), so each tile gives 3 longs of data
        'first, generate pointer to tile

        'calculate vga x assuming 30 cols
        ' (assuming ncols=30)  (sx=i*ncols*16/jx) if ncols=30-> sx=i*2

        'calculate vga y assuming 15 rows   (sy=row*nrows*16/jy) =row*rows*16/160=row*15/10=row*3/2
        mov sy, jrow
        shl sy,#1  'multiply by 2
        add sy,jrow 'and add to effectively multiply by 3
        shr sy,#1 'divide by 2 to get vga y


        'now calc tile #s= position/16
        mov ytile,sy
        shr ytile,#4

        'now calc screen pointer
        mov ptile,#0'ytile  'need to multiply by ncols. 
        mov count,jncols
:mult         'quick and dirty multiply
        add ptile,ytile
        djnz count,#:mult
          'ptile:=screen[ytile * cols + xtile]  'ptile points to 16x16x4 color character + color info
        add ptile,xtile
          'since screen is a word array, need to multiply index by 2
        shl ptile,#1
        add ptile,jpscreen 'Now, ptile points to the correct screen tile... 
        rdword stile,ptile  'get screen tile info

        'need to seperate into color and character parts
        mov pcolor,stile
        shr pcolor,#10  'pcolor is now color index
        shl stile,#6  'stile now points to character data
        and sy,#$0F  'sy is now the line of the character we need
        shl sy,#2   'multiply by 4 to get line offset
        add stile,sy  'stile now points to the line we want
        'get line
        rdlong line,stile
        'the line contains 16 pixels, we used every other one (8 pixels) to form 3 longs of data
        'get colors
        shl pcolor,#2 'multiply by 4 as it is a table of longs
        add pcolor,jpvga_colorbase 'add in base address
        rdlong colors,pcolor  'get the 4 colors

        'init loop
        mov npixels,#8
        mov boffset,#0  'offset to 3 nibble color info in 3 long data
        mov data1,#0
        mov data2,#0
        mov data3,#0
EightPixelLoop
        mov colori,line  'going to shift line 4 bits every loop
        and colori,#3  'rightmost 2 bits are the color#
        shl colori,#3  'multiply by 8 to give amount to shift color by
        mov color,colors
        shr color,colori 'LSByte now contains the color

        'mov color,#%%3000
        
        'extract color info
        mov red, color
        mov green, color
        mov blue, color
        and red,#%%3000
        shl red,#24
        and green, #%%0300
        shl green,#26
        and blue, #%%0030
        shl blue,#28

        
        'now, save 3 nibble colors to output data in the right place
        'first set plong to the long# (0..2) and loffset to the offset to red nibble (0,4,8,12,16,20,24,28)
        mov loffset,boffset  'l3offse
        mov plong,#data1  'init pointer to data output
        cmp loffset,#32 wc, wz
        if_ae sub loffset,#32
        if_ae add plong,#1
        cmp loffset,#32 wc, wz
        if_ae sub loffset,#32
        if_ae add plong,#1
        'save red
        movd orRed,plong
        shr red,loffset
orRed   or 0,red
        'save green
        add loffset,#4
        cmp loffset,#32 wc, wz
        if_ae sub loffset,#32
        if_ae add plong,#1
        movd orGreen,plong
        shr green,loffset
orGreen  or 0,green
        'save blue
        add loffset,#4
        cmp loffset,#32 wc, wz
        if_ae sub loffset,#32
        if_ae add plong,#1
        movd orBlue,plong
        shr blue,loffset
orBlue  or 0,blue        
        '
        'loop
        shr line,#4  'shift line so next pixel is right 2 bits
        add boffset,#12
        djnz npixels,#EightPixelLoop
        
           
        'store 3 longs to output buffer        
        rol data1,#8
        wrbyte data1,y
        add y,#1
        rol data1,#8
        wrbyte data1,y
        add y,#1
        rol data1,#8
        wrbyte data1,y
        add y,#1
        rol data1,#8
        wrbyte data1,y
        add y,#1
        
        rol data2,#8
        wrbyte data2,y
        add y,#1
        rol data2,#8
        wrbyte data2,y
        add y,#1
        rol data2,#8
        wrbyte data2,y
        add y,#1
        rol data2,#8
        wrbyte data2,y
        add y,#1

        rol data3,#8
        wrbyte data3,y
        add y,#1
        rol data3,#8
        wrbyte data3,y
        add y,#1
        rol data3,#8
        wrbyte data3,y
        add y,#1
        rol data3,#8
        wrbyte data3,y
        add y,#1


        'loop
        add xtile,#1
        djnz ntiles,#TileLoop
        
        'wait for next ling
        jmp #waitRow

          
End
        'now stop
        COGID thisCog1
        COGSTOP thisCog1   

'variables for loading parameters (from keyboard.spin)        
x       long 0
y       long 0        
dlsb    long    1 << 9 


'Working variables
ntiles long 30  'horizontal tile loop count, assumes 30 tile column display
'jx long 0 
sx long 0
sy long 0
count long 0
lastrow long 0
jpixel long 0
npixels long 8
ytile long 0
xtile long 0                                                                    
stile long 0
pcolor long 0
ptile long 0
line long 0
colors long 0
colori long 0
color long 0
red long 0
green long 0
blue long 0
color3 long $00000000
data1 long 0
data2 long 0
data3 long 0
boffset long 0
loffset long 0
plong long 0

'input parameters
jrow   long 0 'assembly routine will monitor this        
jpscreen   long 0 
jpvga_colorbase   long 0          
jncols long 0
jBuffer long 0

'variable for stopping cog
thisCog1 LONG 0

        FIT 496  